import * as childProcess from 'child_process';
import * as fs from 'fs';
import * as stream from 'stream';

export default function performDiffing(sourceFile: string, diffStream: stream.Readable, targetFile: string): Promise<void> {
  return new Promise((resolve, reject) => {
    const targetStream = fs.createWriteStream(targetFile);

    //spawn xdelta3 process, set up listeners
    const process = childProcess.spawn('xdelta3', ['-d', '-s', sourceFile]);
    diffStream.pipe(process.stdin);
    process.stdout.pipe(targetStream);
    process.stderr.on('data', (chunk) => {
      reject(`Error during xdelta3: ${chunk}`);
    });

    process.on('error', (error) => {
      reject(`Error during xdelta3: ${error}`);
    });

    process.on('exit', (code, signal) => {
      if (code === 0) {
        resolve();
      } else {
        reject(`xdelta3 exited with code "${code}" due to signal "${signal}".`);
      }
    });
  });
}
